from django.conf.urls import patterns, include, url
from views.admin import AdminView


_urlpatterns = patterns(
    "",

    url(r"^djinn_admin/?$",
        AdminView.as_view(),
        name="djinn_admin"),
)


urlpatterns = patterns(
    '',
    (r'^djinn/', include(_urlpatterns)),
    )



from HTMLParser import HTMLParser
from django.contrib.contenttypes.models import ContentType
from django.conf import settings
from django.core.cache import cache


# URN schema for objects
#
URN_SCHEMA = "urn:pu.in:%(object_app)s:%(object_ctype)s:%(object_id)s"


def _class_implements(clazz, superclazz, check_self=True):

    well_does_it = False

    if check_self and clazz == superclazz:
        return True
    elif isinstance(superclazz, basestring) and superclazz in map(
            lambda x: "%s.%s" % (
                x.__module__.split('.')[0], x.__name__), clazz.__bases__):
        well_does_it = True
    elif superclazz in clazz.__bases__:
        well_does_it = True
    else:
        for base in clazz.__bases__:
            well_does_it = _class_implements(base, superclazz)
            if well_does_it:
                break

    return well_does_it


def implements(instance, clazz):

    """ Is it a bird? A plane? A clazz? """

    return _class_implements(instance.__class__, clazz)


def extends(clazz, otherclazz):

    """ Does the class extend the other one? """

    return _class_implements(clazz, otherclazz, check_self=False)


def object_to_urn(object):

    """ Create A URN for the given object """
    if not object:
        return ""

    app_label = getattr(object, "app_label", object._meta.app_label)
    ct_name = getattr(object, object.ct_name,
                      object.__class__.__name__.lower())

    return URN_SCHEMA % {'object_app': app_label,
                         'object_ctype': ct_name,
                         'object_id': object.id}


def urn_to_ctype_and_id(urn):

    """ Fetch the ctype and object_id for this URN. If not found, return None """

    parts = urn.split(":")

    ctype = ContentType.objects.get(app_label=parts[2], model=parts[3])

    try:
        return ctype, parts[4]
    except:
        return None


def urn_to_object(urn):

    """ Fetch the object for this URN. If not found, return None """

    parts = urn.split(":")

    key = "contenttype-%s-%s" % (parts[2], parts[3])
    ctype = cache.get(key, None)

    if not ctype:
        ctype = ContentType.objects.get(app_label=parts[2], model=parts[3])
        cache.set(key, ctype)

    try:
        return ctype.get_object_for_this_type(id=parts[4])
    except:
        return None


def get_object_by_ctype_id(ctype_id, _id, app_label=None):

    key = "contenttype-%s-%s" % (app_label, ctype_id)

    ctype = cache.get(key, None)

    if not ctype:
        ctype = ContentType.objects.get(app_label=app_label, model=ctype_id)
        cache.set(key, ctype)

    return ctype.get_object_for_this_type(id=_id)


def get_apps(name_filter=None):

    """ TODO: Django 1.7 has this call built in """

    _apps = {}

    for _app in settings.INSTALLED_APPS:
        if not name_filter or name_filter in _app:
            _apps[_app] = __import__(_app)

    return _apps


class HTMLTruncate(HTMLParser):

    """ Truncating class for html fragments """

    def __init__(self, limit):

        HTMLParser.__init__(self)

        self.limit = limit
        self.done = False
        self.count = 0
        self.truncated = []
        self.singles = ["br", "img", "hr"]
        self.queue = []

    def _flatten_attrs(self, attrs):

        if attrs:
            return " " + " ".join(
                ['%s="%s"' % (name, val) for name, val in attrs])
        else:
            return ""

    def handle_starttag(self, tag, attrs):

        if not self.done:

            if tag in self.singles:
                single_marker = "/"
            else:
                single_marker = ""

            self.queue.append(tag)
            self.truncated.append("<%s%s%s>" % (tag,
                                                self._flatten_attrs(attrs),
                                                single_marker))

    def handle_endtag(self, tag):

        if not self.done:
            if tag not in self.singles:
                self.truncated.append("</%s>" % tag)
            self.queue.pop()

    def handle_data(self, data):

        if not self.done:
            if self.count + len(data) > self.limit:
                self.done = True
                data = data[:(self.limit - self.count)]

            self.count += len(data)
            self.truncated.append(data)

    def truncate(self, text):

        self.done = False
        self.count = 0
        self.truncated = []
        self.queue = []

        self.feed(text)

        while self.queue:
            self.truncated.append("</%s>" % self.queue.pop())

        return "".join(self.truncated)



import pkg_resources
from django.views.generic import TemplateView
from django.core.urlresolvers import reverse


class AdminMixin(object):

    """ Mixin for admin views """

    def list_tools(self):

        tools = []

        for entrypoint in pkg_resources.iter_entry_points(group="djinn.tool",
                                                          name="info"):

            tool = entrypoint.load()()

            tool['url'] = reverse(tool['url'])

            tools.append(tool)

        return tools


class AdminView(TemplateView, AdminMixin):

    template_name = "djinn_core/admin.html"



import io
import os
import sys
import pkg_resources
from django.template.loaders.base import Loader as BaseLoader
from django.utils.importlib import import_module
from django.core.exceptions import ImproperlyConfigured
from django.template import Engine
from django.template import TemplateDoesNotExist


fs_encoding = sys.getfilesystemencoding() or sys.getdefaultencoding()

app_template_dirs = []

for entrypoint in pkg_resources.iter_entry_points(group="djinn.skin"):

    app = entrypoint.module_name

    try:
        mod = import_module(app)
    except ImportError, e:
        raise ImproperlyConfigured('ImportError %s: %s' % (app, e.args[0]))

    template_dir = os.path.join(os.path.dirname(mod.__file__), 'templates')

    if os.path.isdir(template_dir):
        app_template_dirs.append(template_dir.decode(fs_encoding))


app_template_dirs = tuple(app_template_dirs)


class SkinTemplateLoader(BaseLoader):
    is_usable = True

    def load_template_source(self, template_name, template_dirs=None):
        tried = []
        for filedir in app_template_dirs:
            filepath = "%s/%s" % (filedir, template_name)
            try:
                with io.open(filepath,
                             encoding=self.engine.file_charset) as fp:
                    return fp.read(), filepath
            except IOError:
                tried.append(filepath)
        if tried:
            error_msg = "Tried %s" % tried
        else:
            error_msg = (
                "Your template directories configuration is empty. "
                "Change it to point to at least one template directory.")
        raise TemplateDoesNotExist(error_msg)
    load_template_source.is_usable = True

_loader = SkinTemplateLoader(Engine())
